package com.example.custom_track

import RetrofitClient
import android.app.AlertDialog
import android.content.ContentValues
import android.content.Context
import android.content.pm.ActivityInfo
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Rect
import android.net.Uri
import android.os.Bundle
import android.provider.MediaStore
import android.util.Log
import android.view.View
import android.widget.EditText
import android.widget.Switch
import android.widget.TextView
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.camera.core.Camera
import androidx.camera.core.CameraSelector
import androidx.camera.core.ImageCapture
import androidx.camera.core.ImageCaptureException
import androidx.camera.core.Preview
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.camera.view.PreviewView
import androidx.core.content.ContextCompat
import com.google.android.material.button.MaterialButton
import com.google.common.util.concurrent.ListenableFuture
import com.google.mlkit.vision.barcode.BarcodeScannerOptions
import com.google.mlkit.vision.barcode.BarcodeScanning
import com.google.mlkit.vision.barcode.common.Barcode
import com.google.mlkit.vision.common.InputImage
import com.google.mlkit.vision.text.Text
import com.google.mlkit.vision.text.TextRecognition
import com.google.mlkit.vision.text.latin.TextRecognizerOptions
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.io.File
import java.io.IOException


class PassActivity: AppCompatActivity() {
    private lateinit var previewView: PreviewView
    private lateinit var switchToggle: Switch
    private lateinit var captureButton: MaterialButton
    private var camera: Camera? = null
    private var imageCapture: ImageCapture? = null
    private lateinit var resultTextView: TextView
    private var charHeight = 0  // Variable to store character height for bounding box
    private var boundingBox = Rect()  // Initially empty
    private lateinit var overlayView: OverlayView
    private lateinit var addInputButton: MaterialButton
    private fun lockOrientation() {
        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
    }

    private fun unlockOrientation() {
        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED
    }




    override fun onCreate(savedInstanceState: Bundle?) {

        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_pass)
        lockOrientation()
        overlayView = findViewById(R.id.overlayView)
        addInputButton = findViewById(R.id.add_input_button)
        previewView = findViewById(R.id.preview_view)
        switchToggle = findViewById(R.id.switch_toggle)
        captureButton = findViewById(R.id.icon_capture)
        resultTextView = findViewById(R.id.mrzResultTextView)
        // Set the default label
        switchToggle.text = "Passport"

        // Add a listener for toggle changes
        switchToggle.setOnCheckedChangeListener { _, isChecked ->
            if (isChecked) {
                // Switch is toggled on
                switchToggle.text = "Barcode"
            } else {

                // Switch is toggled off
                switchToggle.text = "Passport"
            }
        }

        // Check for camera permission and start the camera
        if (ContextCompat.checkSelfPermission(this, android.Manifest.permission.CAMERA) == android.content.pm.PackageManager.PERMISSION_GRANTED) {
            startCamera()
        } else {
            requestPermissions(arrayOf(android.Manifest.permission.CAMERA), 100)
        }
        addInputButton.setOnClickListener { v: View? ->
            // Create an AlertDialog to show input
            val builder: AlertDialog.Builder = AlertDialog.Builder(this)
            val inflater = layoutInflater
            val dialogView: View = inflater.inflate(
                R.layout.dialog_input,

                null
            ) // Create a separate layout for the dialog
            builder.setView(dialogView)

            val userInput =
                dialogView.findViewById<EditText>(R.id.input_field) // Assuming an EditText in dialog_input.xml
            builder.setPositiveButton("OK") { dialog, which ->
                val inputText = userInput.text.toString()
                RetrofitClient.apiService1.getPassengerInformation(inputText).enqueue(object :
                    Callback<PassengerDataDTO> {
                    override fun onResponse(call: Call<PassengerDataDTO>, response: Response<PassengerDataDTO>) {
                        if (response.isSuccessful && response.body()!!.passportNumber!= null) {

                            val apiResponse = response.body()!!
                            val stringBuilder = StringBuilder().apply {
                                append("Departure: ${apiResponse.departure}\n")
                                append("First Name: ${apiResponse.firstName}\n")
                                append("Middle Name: ${apiResponse.middleName}\n")
                                append("Last Name: ${apiResponse.lastName}\n")
                                append("Passport Number: ${apiResponse.passportNumber}\n")
                                append("Risk Value: ${apiResponse.riskValue}\n")
                                append("Risk Level: ${apiResponse.riskLevel}\n")
                                append("Risk Comment: ${apiResponse.riskComment}")
                            }

                            // Set the result to the TextView
                            resultTextView.text = stringBuilder.toString()


                        } else {
                            Toast.makeText(this@PassActivity, "Passport no not found in database", Toast.LENGTH_SHORT).show()
                        }
                    }

                    override fun onFailure(call: Call<PassengerDataDTO>, t: Throwable) {
                        Log.e("APIError", "Error: ${t.message}")
                        Toast.makeText(this@PassActivity, "Network error: ${t.message}", Toast.LENGTH_SHORT).show()
                    }
                })
//                Toast.makeText(this, "Input: $inputText", Toast.LENGTH_SHORT).show()
            }
            builder.setNegativeButton("Cancel") { dialog, which -> dialog.dismiss() }
            builder.create().show()
        }
        captureButton.setOnClickListener {
          if(switchToggle.text=="Passport"){
              val outputOptions =
                  ImageCapture.OutputFileOptions.Builder(createTempFile(applicationContext))
                      .build()

              // Take picture
              imageCapture?.takePicture(outputOptions, ContextCompat.getMainExecutor(this),
                  object : ImageCapture.OnImageSavedCallback {
                      override fun onImageSaved(outputFileResults: ImageCapture.OutputFileResults) {
                          val savedUri = outputFileResults.savedUri
                          val bitmap = BitmapFactory.decodeFile(savedUri?.path)
                          if (bitmap != null) {


                              // Rotate image if necessary
                              val rotatedBitmap = rotateImageIfRequired(bitmap, savedUri ?: return)
//                                    saveImage(rotatedBitmap)
                              val croppedBitmap = cropImage(rotatedBitmap)
                              processImageForOCR(croppedBitmap)
                              unlockOrientation() // Unlock after capturing image


                          }
                      }

                      override fun onError(exception: ImageCaptureException) {
                          Log.e(
                              "Camera",
                              "Image capture failed: ${exception.message}",
                              exception
                          )
                      }
                  })
          }
            if(switchToggle.text=="Barcode"){
                Log.d("Barcode==============================================>","Barcode")
                val outputOptions =
                    ImageCapture.OutputFileOptions.Builder(createTempFile(applicationContext))
                        .build()

                // Take picture
                imageCapture?.takePicture(outputOptions, ContextCompat.getMainExecutor(this),
                    object : ImageCapture.OnImageSavedCallback {
                        override fun onImageSaved(outputFileResults: ImageCapture.OutputFileResults) {
                            val savedUri = outputFileResults.savedUri
                            Log.d("Camera", "Image saved to $savedUri")

                            // Convert saved image to Bitmap
                            val bitmap = BitmapFactory.decodeFile(savedUri?.path)

                            // Crop and process the captured image for OCR
                            if (bitmap != null) {


                                // Rotate image if necessary
                                val rotatedBitmap = rotateImageIfRequired(bitmap, savedUri ?: return)
//

                                processBarcodeImage(rotatedBitmap)


                            }
                        }

                        override fun onError(exception: ImageCaptureException) {
                            Log.e(
                                "Camera",
                                "Image capture failed: ${exception.message}",
                                exception
                            )
                        }
                    })

            }
        }


    }

    private fun processBarcodeImage(bitmap: Bitmap) {
        val rotationDegrees = 0 // Replace with actual rotation if known
        saveImage(bitmap)
        val croppedBitmap = cropBarCodeImage(bitmap)

        // Convert the resized bitmap to InputImage
        val image = InputImage.fromBitmap(croppedBitmap, rotationDegrees)

        saveImage(bitmap) // Save the resized image

        val scanner = BarcodeScanning.getClient(
            BarcodeScannerOptions.Builder()
                .setBarcodeFormats(Barcode.FORMAT_ALL_FORMATS) // Focus on all formats or specify explicitly
                .build()
        )

        scanner.process(image)
            .addOnSuccessListener { barcodes ->
                if (barcodes.isNotEmpty()) {
                    Log.d("BarcodeScanning", "Detected ${barcodes.size} barcodes.")
                    handleBarcodeResult(barcodes)
                } else {
                    Log.d("BarcodeScanning", "No barcode detected in the image.")
                    Toast.makeText(this, "No barcode detected", Toast.LENGTH_SHORT).show()
                }
            }
            .addOnFailureListener { e ->
                Log.e("BarcodeScanning", "Error scanning barcode: ${e.message}", e)
                Toast.makeText(this, "Scanning failed: ${e.message}", Toast.LENGTH_SHORT).show()
            }
    }

    private fun handleBarcodeResult(barcodes: List<Barcode>) {
        val passportNumber = barcodes.joinToString("\n") { barcode ->
            barcode.displayValue.toString()
        }
        RetrofitClient.apiService1.getPassengerInformation(passportNumber).enqueue(object :
            Callback<PassengerDataDTO> {
            override fun onResponse(call: Call<PassengerDataDTO>, response: Response<PassengerDataDTO>) {

                if (response.isSuccessful && response.body() != null) {
                    val apiResponse = response.body()!!
                    val stringBuilder = StringBuilder().apply {
                        append("Departure: ${apiResponse.departure}\n")
                        append("First Name: ${apiResponse.firstName}\n")
                        append("Middle Name: ${apiResponse.middleName}\n")
                        append("Last Name: ${apiResponse.lastName}\n")
                        append("Passport Number: ${apiResponse.passportNumber}\n")
                        append("Risk Value: ${apiResponse.riskValue}\n")
                        append("Risk Level: ${apiResponse.riskLevel}\n")
                        append("Risk Comment: ${apiResponse.riskComment}")
                    }

                    // Set the result to the TextView
                    resultTextView.text = stringBuilder.toString()


                } else {
                    Toast.makeText(this@PassActivity, "Passport no not found in database", Toast.LENGTH_SHORT).show()
                }
            }

            override fun onFailure(call: Call<PassengerDataDTO>, t: Throwable) {
                Log.e("APIError", "Error: ${t.message}")
                Toast.makeText(this@PassActivity, "Network error: ${t.message}", Toast.LENGTH_SHORT).show()
            }
        })


//        // Find the TextView by its ID and set its text
//        val mrzResultTextView = findViewById<TextView>(R.id.mrzResultTextView)
//        mrzResultTextView.text = passportNumber


    }
    private fun saveImage(imageBitmap: Bitmap) {
        // Save the image just like before
        val contentValues = ContentValues().apply {
            put(MediaStore.Images.Media.TITLE, "captured_image")
            put(MediaStore.Images.Media.DESCRIPTION, "Image captured for OCR processing")
            put(MediaStore.Images.Media.MIME_TYPE, "image/jpeg")
            put(MediaStore.Images.Media.DATE_ADDED, System.currentTimeMillis() / 1000)
            put(MediaStore.Images.Media.DATE_MODIFIED, System.currentTimeMillis() / 1000)
        }

        val uri = contentResolver.insert(MediaStore.Images.Media.EXTERNAL_CONTENT_URI, contentValues)

        try {
            val outputStream = uri?.let { contentResolver.openOutputStream(it) }
            outputStream?.use {
                imageBitmap.compress(Bitmap.CompressFormat.JPEG, 100, it)
            }
            Toast.makeText(this, "Image saved successfully", Toast.LENGTH_SHORT).show()
        } catch (e: Exception) {
            e.printStackTrace()
            Toast.makeText(this, "Failed to save image", Toast.LENGTH_SHORT).show()
        }
    }
    private fun processImageForOCR(bitmap: Bitmap) {
        val image = InputImage.fromBitmap(bitmap, 0)

        //saveImage(bitmap) // Save the cropped image

        // Get an instance of the TextRecognizer
        val recognizer = TextRecognition.getClient(TextRecognizerOptions.DEFAULT_OPTIONS)

        // Process the image and extract text
        recognizer.process(image)
            .addOnSuccessListener { visionText ->
                handleOCRResult(visionText, bitmap)
            }
            .addOnFailureListener { e ->
                Toast.makeText(this, "OCR failed: ${e.message}", Toast.LENGTH_SHORT).show()
                Log.e("PassportOCR", "OCR error", e)
            }
    }
    private fun handleOCRResult(visionText: Text, bitmap: Bitmap) {
        val passportNumber = visionText.text
        Log.d("mrzData before", passportNumber)
        RetrofitClient.apiService1.getPassengerInformation(passportNumber).enqueue(object :
            Callback<PassengerDataDTO> {
            override fun onResponse(call: Call<PassengerDataDTO>, response: Response<PassengerDataDTO>) {
                Log.d("api res >>>>>>>>>>>>>>>>> ","${response.body()?.passportNumber}")

                if (response.isSuccessful && response.body()!!.passportNumber!= null) {
                    val apiResponse = response.body()!!
                    val stringBuilder = StringBuilder().apply {
                        append("Departure: ${apiResponse.departure}\n")
                        append("First Name: ${apiResponse.firstName}\n")
                        append("Middle Name: ${apiResponse.middleName}\n")
                        append("Last Name: ${apiResponse.lastName}\n")
                        append("Passport Number: ${apiResponse.passportNumber}\n")
                        append("Risk Value: ${apiResponse.riskValue}\n")
                        append("Risk Level: ${apiResponse.riskLevel}\n")
                        append("Risk Comment: ${apiResponse.riskComment}")
                    }

                    // Set the result to the TextView
                    resultTextView.text = stringBuilder.toString()


                } else {
                    Toast.makeText(this@PassActivity, "No passport number found", Toast.LENGTH_SHORT).show()
                }
            }

            override fun onFailure(call: Call<PassengerDataDTO>, t: Throwable) {
                Log.e("APIError", "Error: ${t.message}")
                Toast.makeText(this@PassActivity, "Network error: ${t.message}", Toast.LENGTH_SHORT).show()
            }
        })
        // Find the TextView by its ID and set its text
//        val mrzResultTextView = findViewById<TextView>(R.id.mrzResultTextView)
//        mrzResultTextView.text = passportNumber
    }

    private fun cropImage(bitmap: Bitmap): Bitmap {


        val previewWidth = bitmap.width
        val previewHeight = bitmap.height

        // Calculate the center of the preview
        val centerX = previewWidth / 2
        val centerY = previewHeight / 2

        // Use the first line of detected text to estimate character height
        val firstLine = resultTextView.text.split("\n").firstOrNull()
        if (firstLine != null && firstLine.isNotEmpty()) {
            // You can adjust this part to get the height from the OCR result
            charHeight = getCharHeight(firstLine)*2
        }

        // Set the bounding box width (e.g., 50% of screen width) and height based on character height
        val boxWidth = (previewWidth * 0.8).toInt() // 70% of the screen width
        val boxHeight = charHeight * 2  // Height for two characters


        // Ensure the bounds are within the image's dimensions to avoid errors
        val cropLeft = centerX - boxWidth / 2
        val cropTop = centerY-60 - boxHeight / 2
        val cropRight = centerX + boxWidth / 2
        val cropBottom = centerY + boxHeight / 2

        // Crop the bitmap using the bounding box's coordinates
        return Bitmap.createBitmap(bitmap, cropLeft, cropTop, cropRight - cropLeft, cropBottom - cropTop)
    }

    private fun cropBarCodeImage(bitmap: Bitmap): Bitmap {

        val previewWidth = bitmap.width
        val previewHeight = bitmap.height

        // Calculate the center of the preview
        val centerX = previewWidth / 2
        val centerY = previewHeight / 2

        // Use the first line of detected text to estimate character height
        val firstLine = resultTextView.text.split("\n").firstOrNull()
        if (firstLine != null && firstLine.isNotEmpty()) {
            // Adjust this part to get the height from the OCR result
            charHeight = getCharHeight(firstLine) * 6
        }

        // Set the bounding box width (e.g., 80% of screen width) and height based on character height
        val boxWidth = (previewWidth * 0.8).toInt()
        val boxHeight = charHeight * 4 // Height for four characters

        // Ensure the bounds are within the image's dimensions to avoid errors
        val cropLeft = centerX - boxWidth / 2
        val cropTop = centerY - 60 - boxHeight / 2
        val cropRight = centerX + boxWidth / 2
        val cropBottom = centerY + boxHeight / 2

        // Crop the bitmap using the bounding box's coordinates
        val croppedBitmap = Bitmap.createBitmap(bitmap, cropLeft, cropTop, cropRight - cropLeft, cropBottom - cropTop)

        // Apply RGB color adjustment to the cropped image
//        val adjustedBitmap = adjustBitmapColor(croppedBitmap, redOffset = 50, greenOffset = -30, blueOffset = 20)

        return croppedBitmap
    }

//    // Function to adjust RGB color values of a Bitmap
//    private fun adjustBitmapColor(bitmap: Bitmap, redOffset: Int, greenOffset: Int, blueOffset: Int): Bitmap {
//        val width = bitmap.width
//        val height = bitmap.height
//        val adjustedBitmap = Bitmap.createBitmap(width, height, bitmap.config)
//
//        for (x in 0 until width) {
//            for (y in 0 until height) {
//                val pixel = bitmap.getPixel(x, y)
//
//                // Extract original color channels
//                val red = (pixel shr 16 and 0xFF) + redOffset
//                val green = (pixel shr 8 and 0xFF) + greenOffset
//                val blue = (pixel and 0xFF) + blueOffset
//
//                // Clamp the values to the valid range (0-255)
//                val newRed = red.coerceIn(0, 255)
//                val newGreen = green.coerceIn(0, 255)
//                val newBlue = blue.coerceIn(0, 255)
//
//                // Recombine the adjusted color values
//                val newPixel = (0xFF shl 24) or (newRed shl 16) or (newGreen shl 8) or newBlue
//                adjustedBitmap.setPixel(x, y, newPixel)
//            }
//        }
//
//        return adjustedBitmap
//    }

    private fun rotateImageIfRequired(bitmap: Bitmap, uri: Uri): Bitmap {
        val rotation = getImageRotation(uri)
        if (rotation != 0) {
            val matrix = android.graphics.Matrix()
            matrix.postRotate(rotation.toFloat())
            return Bitmap.createBitmap(bitmap, 0, 0, bitmap.width, bitmap.height, matrix, true)
        }
        return bitmap
    }

    // Helper function to get the image rotation from the EXIF data
    private fun getImageRotation(uri: Uri): Int {
        val inputStream = contentResolver.openInputStream(uri)
        val exifInterface = android.media.ExifInterface(inputStream!!)
        val orientation = exifInterface.getAttributeInt(android.media.ExifInterface.TAG_ORIENTATION, android.media.ExifInterface.ORIENTATION_NORMAL)

        return when (orientation) {
            android.media.ExifInterface.ORIENTATION_ROTATE_90 -> 90
            android.media.ExifInterface.ORIENTATION_ROTATE_180 -> 180
            android.media.ExifInterface.ORIENTATION_ROTATE_270 -> 270
            else -> 0
        }
    }

    private fun startCamera() {
        // Only call drawBoundingBox if the switch is in the "Passport" state
        drawBoundingBox()
        val cameraProviderFuture: ListenableFuture<ProcessCameraProvider> = ProcessCameraProvider.getInstance(this)

        cameraProviderFuture.addListener({
            val cameraProvider = cameraProviderFuture.get()

            // Create the Preview use case
            val preview = Preview.Builder().build()
            preview.setSurfaceProvider(previewView.surfaceProvider)

            // Create the ImageCapture use case
            imageCapture = ImageCapture.Builder().build()

            val cameraSelector = CameraSelector.Builder().requireLensFacing(CameraSelector.LENS_FACING_BACK).build()

            // Unbind any previous use cases
            cameraProvider.unbindAll()

            // Bind the Preview and ImageCapture use cases to the camera
            camera = cameraProvider.bindToLifecycle(this, cameraSelector, preview, imageCapture)
        }, ContextCompat.getMainExecutor(this))
    }




    // Get the output directory for saving images
    private fun getOutputDirectory(): File {
        val mediaDir = externalMediaDirs.firstOrNull()?.let {
            File(it, resources.getString(R.string.app_name)).apply { mkdirs() }
        }
        return if (mediaDir != null && mediaDir.exists()) mediaDir else filesDir
    }

    companion object {
        private const val FILENAME_FORMAT = "yyyy-MM-dd-HH-mm-ss-SSS"
    }





    private fun drawBoundingBox() {
        previewView.viewTreeObserver.addOnPreDrawListener {
            val previewWidth = previewView.width
            val previewHeight = previewView.height

            // Calculate the center of the preview
            val centerX = previewWidth / 2
            val centerY = previewHeight / 2

            // Use the first line of detected text to estimate character height
            val firstLine = resultTextView.text.split("\n").firstOrNull()
            if (firstLine != null && firstLine.isNotEmpty()) {
                charHeight = getCharHeight(firstLine)
            }

            // Increase the bounding box width and height
            val boxWidth = (previewWidth * 0.8).toInt() // 70% of the screen width
            val boxHeight = charHeight * 2 // Height for two characters

            // Center the bounding box on the screen
            boundingBox.set(
                centerX - boxWidth / 2,  // Left
                centerY - boxHeight / 2, // Top
                centerX + boxWidth / 2,  // Right
                centerY + boxHeight / 2  // Bottom
            )

            // Update overlay view with the centered bounding box
            overlayView.updateBoundingBox(boundingBox)

            true // Allow drawing
        }
    }

    private fun getCharHeight(text: String): Int {
        // Here, we can estimate the height based on the font size and average character height
        // You can adjust this by testing with various fonts or using actual OCR bounding boxes for better accuracy
        return 70  // Example height, adjust based on your OCR text font size
    }
    fun createTempFile(context: Context): File {
        val file = File(context.cacheDir, "temp_image.jpg")
        try {
            if (file.createNewFile()) {
                Log.d("Camera", "File created: ${file.absolutePath}")
            } else {
                Log.d("Camera", "File already exists.")
            }
        } catch (e: IOException) {
            Log.e("Camera", "Error creating file: ${e.message}")
        }
        return file
    }
}